@@ -12,6 +12,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 from .cWrapper import RGFCLib
 from multiprocessing import Process, Queue, cpu_count
+from scipy.spatial.transform import Rotation as R
 
 class textProgBar:
     """
@@ -274,7 +275,26 @@ def sGHist_multi_mP(inVec, mP, SNR=3.0):
     plt.bar(modelVec, np.ones(modelVec.size), color='g',alpha=0.5)
     plt.show()
 
+def scatter3(mat, inFigure = None, returnFigure = False):
+    """ given a matrix input of size 3 x N, it scatters it in 3D
+    """
+    if(inFigure is None):
+        fig = plt.figure()
+        ax = fig.add_subplot(111, projection='3d')
+        figureCnt = 0
+    else:
+        fig, ax, figureCnt = inFigure
+        figureCnt += 1
+    ax.scatter(mat[0], mat[1], mat[2], label = 'Plot ' + str(figureCnt))
+    if(returnFigure):
+        return((fig, ax, figureCnt))
+    else:
+        if(figureCnt>0):
+            plt.legend()
+        plt.show()
+    
 def getTriangularVertices(n,
+                          rotationAngles = [0, 0, 0],
                           phi_start = 0,
                           phi_end = np.pi,
                           plotIt = False):
@@ -285,7 +305,11 @@ def getTriangularVertices(n,
     input argument
     ~~~~~~~~~~~~~~
     n : number of vertices on the sphere
-    phi_start: phi_end is the pitch
+    rotationAngles: input must be a numpy array of three values
+         in radian (0~2pi) and the output will rotate according 
+         to these angles around axes xyz.
+        default:[0, 0, 0]
+    phi_start: 
         default: 0
     phi_end: 
         default: pi
@@ -297,17 +321,16 @@ def getTriangularVertices(n,
     """
     goldenRatio = (1 + 5**0.5)/2
     i = np.arange(0, n)
-    theta = (2 * np.pi / goldenRatio) * i 
+    theta = (2*np.pi / goldenRatio) * i 
     phi = np.arccos(np.linspace(np.cos(phi_end), np.cos(phi_start), n))
     triVecs = np.array([np.cos(theta) * np.sin(phi), 
                         np.sin(theta) * np.sin(phi), 
                         np.cos(phi)])
+    rotMat = R.from_euler('xyz', rotationAngles)
+    triVecs = rotMat.as_matrix() @ triVecs
 
     if(plotIt):
-        fig = plt.figure()
-        ax = fig.add_subplot(111, projection='3d')
-        ax.scatter(triVecs[0], triVecs[1], triVecs[2])
-        plt.show()    
+        scatter3(triVecs)    
 
     return(triVecs)
 
@@ -324,7 +347,6 @@ class multiprocessor:
     def __init__(self, 
         targetFunction,
         inputs,
-        outputIsNumpy = False,
         max_cpu = None,
         showProgress = False):
         """
@@ -335,12 +357,6 @@ class multiprocessor:
                 the targetFunction, such as by syntax:
                     targetFunction(inputs[0])
                 Indeed we actually do this syntax before anything. 
-            outputIsNumpy: If outputIsNumpy is True, then a numpy array will
-                be allocated and filled in with outputs of the queue. This 
-                only works if the output of the targetFunction is numpy array.
-                notice that it cannot be a number, if it isa number you
-                must set the return of the targetFunction to np.array([output])
-                default: False
             max_cpu: max number of allowed cpu
                 default: None
             showProgress: using textProgBar, it shows the progress of 
@@ -348,7 +364,7 @@ class multiprocessor:
                 default: False
         """
         try:
-            targetFunction(inputs[0])
+            funcOutput = targetFunction(inputs[0])
         except:
             print('Running the following syntax raised an exception:')
             print('targetFunction(inputs[0])')
@@ -356,9 +372,26 @@ class multiprocessor:
             print('You need to make inputs mutable and indexable by axis 0')
             exit()
 
+        if(type(inputs).__module__ == np.__name__):
+            self.n_pts = inputs.shape[0]
+            if(showProgress):
+                print('Input is a numpy ndArray')
+        else:
+            try:
+                self.n_pts = len(inputs)
+                if(showProgress):
+                    print('Input is not a numpy ndArray')
+            except:
+                print('I can not get the length of the input list')
+                print('we are supporting lists, tuples and numpy ndarrays')
+                exit()
+
+        if(type(funcOutput).__module__ == np.__name__):
+            self.outputIsNumpy = True
+            if(showProgress):
+                print('output is a numpy ndArray')
         self.targetFunction = targetFunction
         self.inputs = inputs
-        self.outputIsNumpy = outputIsNumpy
         self.showProgress = showProgress
         self.max_cpu = max_cpu
     
@@ -366,24 +399,18 @@ class multiprocessor:
         result = self.targetFunction(inputArgs)
         aQ.put(list([_procID, result]))
 
-    def __call__(self):
-        #for example allData is size N by D
-        try:
-            N = len(self.inputs)
-        except:
-            N = self.inputs.shape[0]
-        
+    def start(self):        
         myCPUCount = cpu_count()-1
         if(self.max_cpu is not None):
             myCPUCount = np.minimum(self.max_cpu, myCPUCount)
         aQ = Queue()
-        numProc = N
+        numProc = self.n_pts
         procID = 0
         numProcessed = 0
         numBusyCores = 0
         if(not self.outputIsNumpy):
             allResults = []
-        Q_procID = np.zeros(N, dtype='uint')
+        Q_procID = np.zeros(self.n_pts, dtype='uint')
         while(numProcessed<numProc):
             if (not aQ.empty()):
                 aQElement = aQ.get()
@@ -393,10 +420,9 @@ class multiprocessor:
                 else:
                     if(numProcessed == 0):
                         allResults = np.zeros(
-                            shape = ((N,) + aQElement[1].shape), 
+                            shape = ((self.n_pts,) + aQElement[1].shape), 
                             dtype = aQElement[1].dtype)
-                    else:
-                        allResults[aQElement[0]] = aQElement[1] 
+                    allResults[aQElement[0]] = aQElement[1] 
                 numProcessed += 1
                 numBusyCores -= 1
                 if(self.showProgress):
